/** \file RTDomainDB.cpp 
*COPYRIGHT (C) 2006  Michele Basso   (basso@dsi.unifi.it)
*                    Marco Romagnoli (romagnoli@control.dsi.unifi.it)
*                    Massimo Vassalli(vassalli@inoa.it)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
*/

#include "RTDomainDB.h"

namespace rtaixml_domain
{
	RTDomainDB::RTDomainDB() 
	{ 
		if(IS_DEBUG) std::cout<<"|| RTDomainDB::RTDomainDB() ||"<<std::endl;
		
		//At this time security policy are taken into account in the sw architecture, but are not
		// implemented; there is only a fake class.
		cpSecure = new CheckPointFake();
		
		//Inizialization of the random number generator.
		srand((unsigned)time(0)); 
		if(IS_DEBUG) { std::cout<<"seed:"<<(unsigned)time(0)<<std::endl; std::fflush(stdout);}

		//... perform necessary instance initializations 
		iBasePortXML 	= DEFAULT_BASE_PORT;
		iMaxPortXML 	= DEFAULT_MAX_PORT;
		if(IS_DEBUG) std::cout<<"RTDomainDB::RTDomainDB(): XML Port interval "<< iBasePortXML <<" - "<< iMaxPortXML <<"."<<std::endl;

		sMasterName = DEFAULT_MASTER_NAME;
		if(IS_DEBUG) std::cout<<"RTDomainDB::RTDomainDB(): sMasterName "<<sMasterName<<std::endl;
		if(IS_DEBUG) std::cout<<"|| End of RTDomainDB::RTDomainDB() ||"<<std::endl;
	}

	RTDomainDB::~RTDomainDB()
	{	
	}
	
	unsigned int RTDomainDB::NumberOfActiveProcesses()
	{
		return vAllActors.size();
	}

	ActorDomainRecord RTDomainDB::GetActor(std::string sSessionId)
	{
		ActorDomainRecord adrEmpty;
		for(unsigned int i = 0; i<vAllActors.size();i++)
			if(vAllActors.at(i).getSessionId()==sSessionId)
				return vAllActors.at(i);;
		return adrEmpty;
	}

	//short: 0 - SlaveName, 1 - SessionId, 2 - TargetName, 3 - (non ammesso)PortXML
	ActorDomainRecord RTDomainDB::GetActor(std::string sValue, short Choice)
	{
		ActorDomainRecord adrEmpty;
		for(unsigned int i = 0; i<vAllActors.size();i++)
			switch(Choice)
			{
				case 0:
					if(vAllActors.at(i).getSlaveName()==sValue)
						return vAllActors.at(i);;
					break;
				case 1:
					if(vAllActors.at(i).getSessionId()==sValue)
						return vAllActors.at(i);;
					break;
				case 2:
					if(vAllActors.at(i).getTargetName()==sValue)
						return vAllActors.at(i);;
					break;
				default:
					return adrEmpty;
			}
		return adrEmpty;
	}

	//XML port and Socket port are alternated.
	int RTDomainDB::GetFreePort()
	{
		if(IS_DEBUG) std::cout << "|| RTDomainDB::GetFreePort ||" << std::endl;
		int iFreePort = iBasePortXML+1;
		if(IS_DEBUG) std::cout << "RTDomainDB::GetFreePort - starting port: " << iFreePort << std::endl;
		if(IS_DEBUG) std::cout << "RTDomainDB::GetFreePort - check for available port - vAllActors.size() = " << vAllActors.size() << std::endl;
		for(int i = 0; (unsigned int)i<vAllActors.size();i++) {
			if(IS_DEBUG) std::cout << "RTDomainDB::GetFreePort - used port ("<< i <<"): " << vAllActors.at(i).getPortXML() << std::endl;
			if(vAllActors.at(i).getPortXML()==iFreePort)
			{
				iFreePort+=2;
				if(IS_DEBUG) std::cout << "RTDomainDB::GetFreePort - new port:" << iFreePort  << std::endl;
				i=-1;
			}
		}
		if(iFreePort>iMaxPortXML) {
			if(IS_DEBUG) std::cout << "RTDomainDB::GetFreePort - No more free ports."  << std::endl;
			iFreePort = 0;
		}
	
		if(IS_DEBUG) std::cout << "RTDomainDB::GetFreePort - choosen port: " << iFreePort  << std::endl;
		if(IS_DEBUG) std::cout << "|| End of RTDomainDB::GetFreePort ||" << std::endl;
		return iFreePort;
	}
	
	
	std::string RTDomainDB::GetFreeSlaveName()
	{
		if(IS_DEBUG) std::cout << "|| RTDomainDB::GetFreeSlaveName ||" << std::endl;
		std::string sRandom=get_random_string(6);
		if(IS_DEBUG) std::cout << "RTDomainDB::GetFreeSlaveName - random slave name: " << sRandom << std::endl;
		if(IS_DEBUG) std::cout << "RTDomainDB::GetFreeSlaveName - check for available slave name - vAllActors.size() = " << vAllActors.size() << std::endl;
		for(int j = 0; j<(int)vAllActors.size();j++) {
			if((vAllActors.at(j).getSlaveName()).compare(sRandom)==0 || IsThereATarget(sRandom))
			{
				sRandom=get_random_string(6);
				if(IS_DEBUG) std::cout << "RTDomainDB::GetFreeSlaveName - new random salve name:" << sRandom  << std::endl;
				j=-1;
			}
		}
		if(IS_DEBUG) std::cout << "RTDomainDB::GetFreeSlaveName - choosen salve name: " << sRandom  << std::endl;
		if(IS_DEBUG) std::cout << "|| End of RTDomainDB::GetFreeSlaveName ||" << std::endl;
		return sRandom;
	}
	
	std::string RTDomainDB::GetFreeSessionId()
	{
		if(IS_DEBUG) std::cout << "|| RTDomainDB::GetFreeSessionId ||" << std::endl;
		std::string sRandom=get_random_string(12);
		if(IS_DEBUG) std::cout << "RTDomainDB::GetFreeSessionId - random session: " << sRandom << std::endl;
		if(IS_DEBUG) std::cout << "RTDomainDB::GetFreeSessionId - check for available session id - vAllActors.size() = " << vAllActors.size() << std::endl;
		for(int j = 0; j<(int)vAllActors.size();j++) {
			if(IS_DEBUG) std::cout << "RTDomainDB::GetFreeSessionId - j = "<< j <<  std::endl;
			if((vAllActors.at(j).getSessionId()).compare(sRandom)==0)
			{
				sRandom=get_random_string(12);
				if(IS_DEBUG) std::cout << "RTDomainDB::GetFreeSessionId - new random session:" << sRandom  << std::endl;
				j=-1;
			}
		}
		if(IS_DEBUG) std::cout << "RTDomainDB::GetFreeSessionId - choosen session id: " << sRandom  << std::endl;
		if(IS_DEBUG) std::cout << "|| End of RTDomainDB::GetFreeSessionId ||" << std::endl;
		return sRandom;
	}
		
	bool RTDomainDB::AddActor(ActorDomainRecord adrNew)
	{
		if(IS_DEBUG) std::cout << "|| RTDomainDB::AddActor ||" << std::endl;
		if(IS_DEBUG) std::cout << "RTDomainDB::AddActor - Checking the actor existence - vAllActors.size()="<< vAllActors.size() << std::endl;
		for(unsigned int i = 0; i<vAllActors.size();i++) {
			if(IS_DEBUG) std::cout << "RTDomainDB::AddActor - Actor n."<< i << std::endl;
			if(IS_DEBUG) std::cout << "RTDomainDB::AddActor - SlaveName: "<< vAllActors.at(i).getSlaveName() << "<->" << adrNew.getSlaveName() << std::endl;
			if(IS_DEBUG) std::cout << "RTDomainDB::AddActor - SessionID: "<< vAllActors.at(i).getSessionId() << "<->" << adrNew.getSessionId() << std::endl;
			if(IS_DEBUG) std::cout << "RTDomainDB::AddActor - TargetName: "<< vAllActors.at(i).getTargetName() << "<->" << adrNew.getTargetName() << std::endl;
			if(IS_DEBUG) std::cout << "RTDomainDB::AddActor - XML Port: "<< vAllActors.at(i).getPortXML() << "<->" << adrNew.getPortXML() << std::endl;
			if(adrNew.getSlaveName()==vAllActors.at(i).getSlaveName() ||
				adrNew.getSessionId()==vAllActors.at(i).getSessionId() ||
				adrNew.getTargetName()==vAllActors.at(i).getTargetName() ||
				adrNew.getPortXML()==vAllActors.at(i).getPortXML())
			{
				if(IS_DEBUG) std::cout << "RTDomainDB::AddActor - vAllActors.size()="<< vAllActors.size() << std::endl;
				if(IS_DEBUG) std::cout << "RTDomainDB::AddActor - The actor is already registered in the rtaixml domain." << std::endl;
				if(IS_DEBUG) std::cout << "|| End of RTDomainDB::AddActor ||" << std::endl;
				return false;
			}
		}
		if(IS_DEBUG) std::cout << "RTDomainDB::AddActor - Inserting new actor in the rtaixml domain..." << std::endl;
		vAllActors.push_back(adrNew);
		if(IS_DEBUG) std::cout << "RTDomainDB::AddActor - ...done" << std::endl;
		if(IS_DEBUG) std::cout << "RTDomainDB::AddActor - vAllActors.size()="<< vAllActors.size() << std::endl;
		if(IS_DEBUG) std::cout << "|| End of RTDomainDB::AddActor ||" << std::endl;
		return true;
	}

	bool RTDomainDB::AddActorAndChoosePort(ActorDomainRecord adrNew)
	{
		adrNew.setPortXML(GetFreePort());
		return AddActor(adrNew);
	}

	//in order:  sSlaveName sSessionId sTargetName
	bool RTDomainDB::AddNewActorAndChoosePort(	std::string sTargetName_new)
	{
		if(IS_DEBUG) std::cout << "|| RTDomainDB::AddNewActorAndChoosePort ||" << std::endl;
		if(CheckTargetSlaveManager(sTargetName_new))
		{
			if(IS_DEBUG) std::cout << "RTDomainDB::AddNewActorAndChoosePort - Target: "<< sTargetName_new << " already managed by "<< getSlaveName(sTargetName_new,0) << "slave server." << std::endl;
			//If multiple slave for one target not allowed!
			if(!cpSecure->MultipleSlaveOneTarget()) {
				if(IS_DEBUG) std::cout << "RTDomainDB::AddNewActorAndChoosePort - Multiple target management NOT allowed." << std::endl;
				if(IS_DEBUG) std::cout << "|| End of RTDomainDB::AddNewActorAndChoosePort ||" << std::endl;
				return false;
			}
			else
			{
				if(IS_DEBUG) std::cout << "RTDomainDB::AddNewActorAndChoosePort - Multiple target management allowed!" << std::endl;
			}
		}
		ActorDomainRecord adrNew(GetFreeSlaveName(), GetFreeSessionId(), sTargetName_new,GetFreePort());
		if(IS_DEBUG) std::cout << "|| End of RTDomainDB::AddNewActorAndChoosePort ||" << std::endl;
		return AddActor(adrNew);
	}
	

	bool RTDomainDB::RemoveActor(ActorDomainRecord adrRemove)
	{
		if(IS_DEBUG) std::cout << "|| RTDomainDB::RemoveActor ||" << std::endl;
		if(IS_DEBUG) std::cout << "Target: " << adrRemove.getTargetName() << " and Slave server: " << adrRemove.getSlaveName() << std::endl;
		for(unsigned int i = 0; i<vAllActors.size();i++)
			if(adrRemove.getSlaveName()==vAllActors.at(i).getSlaveName() &&
				adrRemove.getSessionId()==vAllActors.at(i).getSessionId() &&
				adrRemove.getTargetName()==vAllActors.at(i).getTargetName() &&
				adrRemove.getPortXML()==vAllActors.at(i).getPortXML())
			{
				std::vector<ActorDomainRecord>::iterator the_iterator;
				the_iterator = vAllActors.begin();
				vAllActors.erase(the_iterator+i);
				if(IS_DEBUG) std::cout << "Target: " << adrRemove.getTargetName() << " found!" << std::endl;
				if(IS_DEBUG) std::cout << "|| End of RTDomainDB::RemoveActor ||" << std::endl;
				return true;
			}
		if(IS_DEBUG) std::cout << "Target: " << adrRemove.getTargetName() << " NOT present!" << std::endl;
		if(IS_DEBUG) std::cout << "|| End of RTDomainDB::RemoveActor ||" << std::endl;
		return false;
	}

	bool RTDomainDB::RemoveActor(std::string sTaskNameToRemove)
	{
		if(IS_DEBUG) std::cout << "|| RTDomainDB::RemoveActor ||" << std::endl;
		for(unsigned int i = 0; i<vAllActors.size();i++)
			if(sTaskNameToRemove==vAllActors.at(i).getTargetName())
			{
				std::vector<ActorDomainRecord>::iterator the_iterator;
				the_iterator = vAllActors.begin();
				vAllActors.erase(the_iterator+i);
				if(IS_DEBUG) std::cout << "Target: " << sTaskNameToRemove << " found!" << std::endl;
				if(IS_DEBUG) std::cout << "|| End of RTDomainDB::RemoveActor ||" << std::endl;
				return true;
			}
		if(IS_DEBUG) std::cout << "Target: " << sTaskNameToRemove << " NOT present!" << std::endl;
		if(IS_DEBUG) std::cout << "|| End of RTDomainDB::RemoveActor ||" << std::endl;
		return false;
	}

	bool RTDomainDB::RemoveAllActor(ActorDomainRecord)
	{
		vAllActors.empty();
		return true;
	}
	
	std::string RTDomainDB::getSessionId(std::string sTaskName)
	{
		if(IS_DEBUG) std::cout << "|| RTDomainDB::getSessionId ||" << std::endl;
		if(IS_DEBUG) std::cout << "RTDomainDB::getSessionId - check for session id - vAllActors.size() = " << vAllActors.size() << std::endl;
		if(IS_DEBUG) std::cout << "RTDomainDB::getSessionId - Target name to find: " << sTaskName << std::endl;
		for(unsigned int i = 0; i<vAllActors.size();i++) {
			if(IS_DEBUG) std::cout << "RTDomainDB::getSessionId - session id ("<< i <<"): " << vAllActors.at(i).getSessionId() << std::endl;
			if(IS_DEBUG) std::cout << "RTDomainDB::getSessionId - target name ("<< i <<"): " << vAllActors.at(i).getTargetName() << std::endl;
			if(sTaskName==vAllActors.at(i).getTargetName())
			{
				if(IS_DEBUG) std::cout << "Found - Session ID: " << vAllActors.at(i).getSessionId() << std::endl;
				if(IS_DEBUG) std::cout << "|| End of RTDomainDB::getSessionId ||" << std::endl;
				return vAllActors.at(i).getSessionId();
			}
		}
		if(IS_DEBUG) std::cout << "NOT Found!" << std::endl;
		if(IS_DEBUG) std::cout << "|| End of RTDomainDB::getSessionId ||" << std::endl;
		return "";

	}
	
	//0 TaskName, 1 SessionId
	std::string RTDomainDB::getSlaveName(std::string sName,short sChoice)
	{
		for(unsigned int i = 0; i<vAllActors.size();i++)
			if( (sName==vAllActors.at(i).getTargetName() && sChoice == 0) ||
				(sName==vAllActors.at(i).getSessionId() && sChoice == 1) )
			{
				return vAllActors.at(i).getSlaveName();
			}
		return "";
	}

	int RTDomainDB::getPortXMLSlave(std::string sName, short sChoice)
	{
		if(IS_DEBUG) { std::cout<<"RTDomainDB::getPortXMLSlave(): Name: "<< sName << " - Choice: "<< sChoice << std::endl;std::fflush(stdout); }
		if(IS_DEBUG) { std::cout<<"RTDomainDB::getPortXMLSlave(): vAllActors.size(): "<< vAllActors.size() << std::endl;std::fflush(stdout); }
		for(unsigned int i = 0; i<vAllActors.size();i++)
		{
			if(IS_DEBUG) { std::cout<<"RTDomainDB::getPortXMLSlave(): i: "<< i << std::endl;std::fflush(stdout); }
			if( (sName==vAllActors.at(i).getTargetName() && sChoice == 0) ||
				(sName==vAllActors.at(i).getSessionId() && sChoice == 1) )
			{
				return vAllActors.at(i).getPortXML();
			}
		}
		if(IS_DEBUG) { std::cout<<"RTDomainDB::getPortXMLSlave(): No Actors!"<< std::endl;std::fflush(stdout); }
		return 0;
	}
	
	std::vector< SlaveInfo > RTDomainDB::getAllSlaveInfo()
	{
		std::vector< SlaveInfo > vSlaveInfos;
		for(unsigned int i = 0; i<vAllActors.size();i++) {
			SlaveInfo appo( vAllActors.at(i).getSlaveName(), vAllActors.at(i).getPortXML() ); 
			vSlaveInfos.push_back( appo );
		}
		return vSlaveInfos;
	}
	
		
	std::string RTDomainDB::getTaskName(std::string sSessionId)
	{
		for(unsigned int i = 0; i<vAllActors.size();i++)
			if(sSessionId==vAllActors.at(i).getSessionId())
			{
				return vAllActors.at(i).getTargetName();
			}
		return "";
	}

	void RTDomainDB::PrintCurrentStatus()
	{
		std::cout << "================================================================================" << std::endl;
		std::cout << "\t\t\t INFO ABOUT RTAIXML DOMAIN :\t\t" << std::endl;
		std::cout << "================================================================================" << std::endl;
		std::cout << "Master Server Name :\t\t" << sMasterName << std::endl;
		std::cout << "XML master communication port :\t" << iBasePortXML << std::endl;
		std::cout << "XML slave starting port :\t" << iBasePortXML+1 << std::endl;
		std::cout << "XML slave port interval:\t" << iMaxPortXML-iBasePortXML << std::endl;
		std::cout << "Security policy :\t\t" << cpSecure->StringPrintPolicy() << std::endl;
		std::cout << std::endl;
		std::cout << "Number of running processes :\t" << NumberOfActiveProcesses() << std::endl;
		if((short)NumberOfActiveProcesses()>0)
		{
		std::cout << "--------------------------------------------------------------------------------" << std::endl;
		std::cout << "Slave Server Name\tSession ID\t\tTarget Name\t\tXML Port" << std::endl;
		std::cout << "--------------------------------------------------------------------------------" << std::endl;
		for(unsigned short i=0;i<NumberOfActiveProcesses();i++)
		{
		std::cout << vAllActors.at(i).getSlaveName() << "\t\t\t" << vAllActors.at(i).getSessionId() << "\t\t" << vAllActors.at(i).getTargetName() << "\t\t\t" << vAllActors.at(i).getPortXML() << std::endl;
		}
		std::cout << "--------------------------------------------------------------------------------" << std::endl;
		}
		std::cout << "================================================================================" << std::endl;
	}

	bool RTDomainDB::CheckTargetSlaveManager(std::string sTargetName_tocheck) 
	{
		return !(getSlaveName(sTargetName_tocheck,0).empty());
	}

	int RTDomainDB::ClearDomain() 
	{
		int iDisabledTargetFound = 0;
		std::string sTN;
		for(unsigned int i = 0; i<vAllActors.size();i++) {
			sTN = vAllActors.at(i).getTargetName();
			if( sTN.compare("")==0 && !IsThereATarget(sTN) ) {
				iDisabledTargetFound++;
				vAllActors.at(i).setTargetName("");
			}
		}
		return iDisabledTargetFound;
	}

	bool RTDomainDB::IsTargetPresent(std::string sTN)
	{
		for(unsigned int i = 0; i<vAllActors.size();i++)
			if( sTN==vAllActors.at(i).getTargetName())
				return true;
		return false;
	}


} //end namespace rtaixml_domain
