/** \file slave.h  
*COPYRIGHT (C) 2006  Michele Basso   (basso@dsi.unifi.it)
*                    Marco Romagnoli (romagnoli@dsi.unifi.it)
*                    Massimo Vassalli(massimo.vassalli@isc.cnr.it)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
*/


#ifndef _RTAIXML_H_
#define _RTAIXML_H_

namespace rtaixml_domain {


#define RTAIXML_VERSION				"0.8.2"

#define MAX_NHOSTS				100
#define MAX_NAMES_SIZE				256
#define MAX_DATA_SIZE				30
#define MAX_RTAI_SCOPES				20
#define MAX_RTAI_LOGS				20
#define MAX_RTAI_METERS				20
#define MAX_RTAI_LEDS				20
#define MAX_RTAI_SYNCHS				20
#define MAX_BATCH_PARAMS			1000
#define MAX_TRACES_PER_SCOPE			10

#define MAX_MSG_MARGIN				100
#define WAIT_MAX				10
#define WAIT_MIN				1
//Serve il rigo seguente perch questa define  nelle netrpc
#define MAX_MSG_SIZE					1500 
#define MAX_MSG_LEN				(MAX_MSG_SIZE - MAX_MSG_MARGIN)
#define REFRESH_RATE				0.05

#define msleep(t)						do { poll(0, 0, t); } while (0)

typedef struct Target_Parameters_Struct Target_Parameters_T;
typedef struct Target_Blocks_Struct Target_Blocks_T;
typedef struct Target_Scopes_Struct Target_Scopes_T;
typedef struct Target_Logs_Struct Target_Logs_T;
typedef struct Target_ALogs_Struct Target_ALogs_T;
typedef struct Target_Leds_Struct Target_Leds_T;
typedef struct Target_Meters_Struct Target_Meters_T;
typedef struct Target_Synchs_Struct Target_Synchs_T;
typedef struct Batch_Parameters_Struct Batch_Parameters_T;

typedef struct Preferences_Struct Preferences_T;
typedef struct Profile_Struct Profile_T;
typedef struct W_Geometry_Struct W_Geometry_T;
typedef struct RGB_Color_Struct RGB_Color_T;

typedef struct Args_Struct Args_T;
typedef struct Args_Struct_ALog Alog_T;

typedef struct s_idx_T {
	int scope_idx;
	int trace_idx;
};

typedef struct p_idx_T {
	int block_idx;
	int param_idx;
	int val_idx;
};
	
	
struct Args_Struct
{
	const char *mbx_id;
	int index;
	int x, y, w, h;
};

struct Args_Struct_ALog
{
	const char *mbx_id;
	int index;
	int x, y, w, h;
	const char *alog_name;
};

struct Target_Parameters_Struct
{
	char model_name[MAX_NAMES_SIZE];
	char block_name[MAX_NAMES_SIZE];
	char param_name[MAX_NAMES_SIZE];
	unsigned int n_rows;
	unsigned int n_cols;
	unsigned int data_type;
	unsigned int data_class;
	double data_value[MAX_DATA_SIZE];
};

struct Target_Blocks_Struct
{
	char name[MAX_NAMES_SIZE];
	int offset;
};

struct Target_Scopes_Struct
{
	char name[MAX_NAMES_SIZE];
	int ntraces;
	int visible;
	float dt;
};

struct Target_Logs_Struct
{
	char name[MAX_NAMES_SIZE];
	int nrow;
	int ncol;
	//Sampling time (maybe)
	float dt;
};

struct Target_ALogs_Struct
{
	char name[MAX_NAMES_SIZE];
	int nrow;
	int ncol;
	float dt;
};

struct Target_Leds_Struct
{
	char name[MAX_NAMES_SIZE];
	int n_leds;
	int visible;
	float dt;
};

struct Target_Meters_Struct
{
	char name[MAX_NAMES_SIZE];
	int visible;
	float dt;
};

struct Target_Synchs_Struct
{
	char name[MAX_NAMES_SIZE];
	int visible;
	float dt;
};

struct Batch_Parameters_Struct
{
	int index;
	int mat_index;
	double value;
};

struct Preferences_Struct
{
	char *Target_Interface_Task_Name;
	char *Target_Scope_Mbx_ID;
	char *Target_Log_Mbx_ID;
	char *Target_ALog_Mbx_ID;
	char *Target_Led_Mbx_ID;
	char *Target_Meter_Mbx_ID;
	char *Target_Synch_Mbx_ID;
	int MW_w, MW_h, MW_x, MW_y;
	char *Current_Profile;
	int Num_Profiles;
};

typedef enum {
	rt_SCALAR,
	rt_VECTOR,
	rt_MATRIX_ROW_MAJOR,
	rt_MATRIX_COL_MAJOR,
	rt_MATRIX_COL_MAJOR_ND
} Param_Class;

typedef enum {
	CONNECT_TO_TARGET,
	CONNECT_TO_TARGET_WITH_PROFILE,
	DISCONNECT_FROM_TARGET,
	START_TARGET,
	STOP_TARGET,
	UPDATE_PARAM,
	GET_TARGET_TIME,
	BATCH_DOWNLOAD,
	GET_PARAMS,
	CLOSE
} Commands;

typedef enum {
	GEOMETRY_PREF,
	CONNECT_PREF,
	PROFILES_PREF,
	PROFILE_PREF
} Pref_Type;

typedef enum {
	PARAMS_MANAGER,
	SCOPES_MANAGER,
	LOGS_MANAGER,
	ALOGS_MANAGER,
	LEDS_MANAGER,
	METERS_MANAGER,
	SYNCHS_MANAGER
} Manager_Type;

} //end namespace rtaixml_domain

#endif
